 
#include "plasma-tvgr.h"
#include "stationwidget.h"
#include <QPainter>
#include <QFontMetrics>
#include <QSizeF>
#include <QPalette>
#include <QWidgetAction>
#include <QListView>

#include <kio/job.h>
#include <KDialog>
#include <KGlobalSettings>
#include <KStandardDirs>
 
#include <plasma/svg.h>
#include <plasma/theme.h>
#include <math.h>

#define MIN(a, b) (((a) < (b)) ? (a) : (b))
#define MAX(a, b) (((a) > (b)) ? (a) : (b))



PlasmaTvGr::PlasmaTvGr(QObject *parent, const QVariantList &args)
    : Plasma::Applet(parent, args),
    m_date1(),m_date2(),m_dialog(0),m_job(0),m_timer(0),m_loopTimer(0),
    m_oldPixmap(0),m_newPixmap(0)
{
    QTextCodec *codec = QTextCodec::codecForName("ISO 8859-7");
    chan << codec->toUnicode("1");
    chan << codec->toUnicode(""); 
    chan << codec->toUnicode("3");
    chan << codec->toUnicode("Mega"); 
    chan << codec->toUnicode("Ant1");
    chan << codec->toUnicode("Alpha"); 
    chan << codec->toUnicode("Star");
    chan << codec->toUnicode("Alter"); 
    chan << codec->toUnicode(""); 
    chan << codec->toUnicode("+");
    chan << codec->toUnicode("+");
    chan << codec->toUnicode("+");
    chan << codec->toUnicode("MTV");
    chan << codec->toUnicode("NovaCinema1");
    chan << codec->toUnicode("NovaCinema2");
    chan << codec->toUnicode("NovaSports1");
    
    setBackgroundHints(DefaultBackground);
    setHasConfigurationInterface(true);
    QString s1=KStandardDirs::installPath("data");
    s1=s1+"plasmatvgr/";
    kDebug() << "Svgs path:" << s1 <<"\n";
    int w1;
    QStringList filenames;
    filenames << "et1.svg" << "net.svg" << "et3.svg" << "mega.svg" << "ant1.svg" << "alpha.svg" << "star.svg" 
	      << "alter.svg"<< "skai.svg" << "cineplus.svg" << "prismaplus.svg" << "sportplus.svg"<<"mtv.svg"<<
	      "novacinema1.svg"<<"novacinema2.svg"<<"novasports1.svg";
    for(w1=0;w1<16;w1++) 
    {
    m_images.append(new Plasma::Svg(this));
    m_images.last()->setImagePath(s1+filenames[w1]);
    //kDebug()<< "path:"<<m_images.last()->imagePath()<<" valid:"<<m_images.last()->isValid()<<  "\n";
    };
    setAspectRatioMode(Plasma::IgnoreAspectRatio);
    setMinimumSize(48,48);
    setMaximumSize(400,400);
    m_pixmapUpdate=1;
    m_fromConstraints=0;
    m_animID=0;
    m_animOpacity=1.00;
    m_oldPixmap=new QPixmap(size().width(),size().height());
    m_oldPixmap->fill(QColor(Qt::transparent));
    //kDebug()<<m_font.family() <<","<<m_font.pixelSize();
    //kDebug()<<"colfg:"<<m_colfg<<" colsel:"<<m_colselfg;
    m_httpOK=false;
}

void PlasmaTvGr::initMenu()
{
    menu = new KMenu();
    menu->addTitle("Select:");
    for(int w1=0;w1<m_images.size();w1++)
    {
    	QWidgetAction *widgetAction=new QWidgetAction(menu);
	widgetAction->setText(chan[w1]);
	StationWidget* w=new StationWidget(); 
	QPixmap pix(m_images[w1]->pixmap("layer1"));
	w->setPixmap(pix);
	w->setScaledContents(true);
	widgetAction->setDefaultWidget(w);
	menu->addAction(widgetAction);
    };
}
void PlasmaTvGr::execMenu()
{ 
    QAction* act=menu->exec(QCursor::pos());
    if(!act) return;
    int w1,w2=0;
    for(w1=0;w1<chan.size();w1++)
    {
       if(act->text().replace("&","")==chan[w1]) w2=w1;
    };
    selectChannel(w2);
    m_pixmapUpdate=1;
    update();
};

void PlasmaTvGr::mousePressEvent ( QGraphicsSceneMouseEvent * event )
{
	if(signalsBlocked()) 
	{
		Plasma::Applet::mousePressEvent(event);
		return;
	};
	if(event->button()==Qt::LeftButton)
	{
	}else Plasma::Applet::mousePressEvent(event);
};

void PlasmaTvGr::mouseDoubleClickEvent ( QGraphicsSceneMouseEvent * event )
{
	Q_UNUSED(event);	
	execMenu();
};


void PlasmaTvGr::themeUpdated()
{
    KColorScheme plasmaColorTheme = KColorScheme(QPalette::Active, KColorScheme::View, 
						  Plasma::Theme::defaultTheme()->colorScheme());
    m_colfg = plasmaColorTheme.foreground(KColorScheme::InactiveText).color();
    m_colselfg = plasmaColorTheme.foreground(KColorScheme::ActiveText).color();
    if(m_colfg==m_colselfg) {m_colselfg=m_colselfg.lighter(150);m_colfg=m_colfg.darker(150);};
    //m_font = KGlobalSettings::smallestReadableFont();
    m_font=Plasma::Theme::defaultTheme()->font(Plasma::Theme::DefaultFont);
    m_pixmapUpdate=1;
    m_fromConstraints=1;
    update();
};
 
 
PlasmaTvGr::~PlasmaTvGr()
{
    if (hasFailedToLaunch()) {
        // Do some cleanup here
    } else {
        // Save settings
	configAccepted();
    };
    delete m_timer;
    delete m_loopTimer;
    delete menu;
    QListWidgetItem* item;
    while(ui.list1->count())
    {
       item=ui.list1->takeItem(0);
       delete item;
    };
   if(m_newPixmap!=0) delete m_newPixmap;
   if(m_oldPixmap!=0) delete m_oldPixmap;
   delete m_dialog;
};



void PlasmaTvGr::init()
{
    // A small demonstration of the setFailedToLaunch function
    //if (m_icon.isNull()) {
     //   setFailedToLaunch(true, "No world to say hello");
    //}
    m_timer=new QTimer(this);
    connect(m_timer, SIGNAL(timeout()), this, SLOT(updateLists()));
    m_timer->start(60000); //1 min for now
    m_loopTimer=new QTimer(this);
    connect(m_loopTimer, SIGNAL(timeout()), this, SLOT(loopUpdate()));
    
    m_buttonNext = new Plasma::PushButton(this);
    m_buttonNext->setText(">");
    m_buttonNext->setMinimumSize(1,1);
    connect(m_buttonNext, SIGNAL( clicked() ), this, SLOT (nextClicked() ) );
    m_buttonNext->setVisible(true);
    m_buttonPrev = new Plasma::PushButton(this);
    m_buttonPrev->setText("<");
    m_buttonPrev->setMinimumSize(1,1);
    connect(m_buttonPrev, SIGNAL( clicked() ), this, SLOT (prevClicked() ) );
    m_buttonPrev->setVisible(true);

    KConfigGroup cg = config();
    m_loop = cg.readEntry("LoopSeconds", "0").toInt();
    m_channelID = cg.readEntry("SelectedChannel", "0").toInt();
    m_fontSize = cg.readEntry("FontSize", "12").toInt();
    m_chanList = cg.readEntry("ChannelList","1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1");
    connect(Plasma::Theme::defaultTheme(), SIGNAL(themeChanged()), SLOT(themeUpdated()));
    themeUpdated();
    initMenu();
    initConfig();
    m_channel=chan[m_channelID];
    //kDebug() << "LoopSeconds:" << m_loop << "SelectedChannel:"<<m_channelID<<" ChannelName:"<<m_channel <<"\n";
    if(m_loop>0) m_loopTimer->start(m_loop*1000); 
    updateLists();
    update();
} 

void PlasmaTvGr::initConfig()
{
    if (m_dialog == 0)
    {
        m_dialog = new KDialog;
	ui.setupUi(m_dialog->mainWidget());
	m_dialog->setButtons(KDialog::Ok | KDialog::Cancel);
        for(int w1=0;w1<chan.count();w1++)
        {
           ui.list1->addItem(new QListWidgetItem(chan[w1]));
        };
    }
    connect(m_dialog, SIGNAL(okClicked()), this, SLOT(configAccepted()));
    int w1,w2;
    for(w1=0;w1<=ui.fontBox->count();w1++)
    {
      w2=ui.fontBox->itemText(w1).toInt();
      if(w2==m_fontSize) break;
    };
    ui.fontBox->setCurrentIndex(w1);
    switch (m_loop){
    case 0:
      w1=0;break;
    case 15:
      w1=1;break;
    case 30:
      w1=2;break;
    case 60:
      w1=3;break;
    case 600:
      w1=4;break;
    case 1800:
      w1=5;break;
    default:
      w1=0;
    };
    ui.loopBox->setCurrentIndex(w1);

    QStringList chanList=m_chanList.split(",");
    QListWidgetItem* item;
    for(w1=0;w1<chan.count();w1++)
    {
	item=ui.list1->item(w1);
        item->setSelected(chanList[w1].toInt());
    };
};


void PlasmaTvGr::prevClicked()
{
  if(m_loop!=0){ m_loopTimer->stop(); m_loopTimer->start(m_loop*1000);};
  int w1=0;
  while(w1==0)
  {
  if(m_channelID==0) m_channelID=chan.size()-1; else m_channelID--;
  w1 = m_chanList.split(",")[m_channelID].toInt();
  };
  selectChannel(m_channelID);
  m_pixmapUpdate=1;
  update();
};

void PlasmaTvGr::nextClicked()
{
  if(m_loop!=0){ m_loopTimer->stop(); m_loopTimer->start(m_loop*1000);};
  int w1=0;
  while(w1==0)
  {
  if(m_channelID==chan.size()-1) m_channelID=0; else m_channelID++;
  w1 = m_chanList.split(",")[m_channelID].toInt();
  };
  selectChannel(m_channelID);
  m_pixmapUpdate=1;
  update();
};

void PlasmaTvGr::loopUpdate()
{
  nextClicked();
};

void PlasmaTvGr::calculateSizes(void)
{
  QSizeF o_size=contentsRect().size();
  QSizeF n_size;
  double l1,r1,t1,b1;
  int w1;
  double d1;
  if( o_size.width() == 0 || o_size.height() == 0 )
  {
       return;
  };
  getContentsMargins(&l1,&r1,&t1,&b1);
  n_size.setWidth(o_size.width()+l1+r1);
  n_size.setHeight(o_size.height()+b1+t1);
  m_font.setPixelSize(m_fontSize);
  QFontMetrics metric(m_font);
  m_textH=metric.height();
  if(formFactor()==Plasma::Horizontal) 
  {
    d1=MIN(n_size.width(),n_size.height()*130.0/120.0*2.6);
    for(w1=0;w1<=m_images.size()-1;w1++)
    {
      m_images[w1]->resize(d1,d1*120.0/130.0);
    };
    d1=n_size.height()/1.5;if(d1>32) d1=32;
    m_buttonPrev->setGeometry(QRectF(contentsRect().topLeft().x(),contentsRect().topLeft().y(),d1,d1));
    m_buttonNext->setGeometry(QRectF( contentsRect().topRight().x()-d1,contentsRect().top(),d1,d1));
    m_textBox=contentsRect().adjusted(d1,0,-d1,0);
    m_numOfLines=m_textBox.height()/m_textH;
    
  } else if(formFactor()==Plasma::Vertical)
  {
    d1=MIN(n_size.width()*120.0/130.0*2.6,n_size.height());
    for(w1=0;w1<=m_images.size()-1;w1++)
    {
      m_images[w1]->resize(d1,d1*120.0/130.0);
    };
    d1=n_size.width()/1.5;if(d1>32) d1=32;
    m_buttonPrev->setGeometry(QRectF(contentsRect().left(),contentsRect().top(),d1,d1));
    m_buttonNext->setGeometry(QRectF(contentsRect().left(),contentsRect().bottom()-d1,d1,d1));
    m_textBox=contentsRect().adjusted(0,d1,0,-d1);
    m_numOfLines=m_textBox.width()/m_textH;
  } else //all other forms AND planar 
  {
    if (n_size.width()>n_size.height()) d1=n_size.width()/3.0; else d1=n_size.width()/2.0;
    for(w1=0;w1<=m_images.size()-1;w1++)
    {
      m_images[w1]->resize(d1,d1*120.0/130.0);
    };
    if(m_images.size()>0)
    {
    d1=m_images[0]->size().height()/3.0;
    m_buttonPrev->setGeometry(QRectF( contentsRect().topLeft().x(),contentsRect().topLeft().y(),d1,d1));
    m_buttonNext->setGeometry(QRectF( contentsRect().topRight().x()-d1,contentsRect().top(),d1,d1));
    m_textBox=contentsRect().adjusted(0,d1,0,0);
    m_numOfLines=m_textBox.height()/m_textH;
    
    };
   };
  //kDebug() <<"lines:"<< m_numOfLines;
  //kDebug() <<"w:"<<n_size.width()<<",h:"<<n_size.height()<<" form:"<<formFactor();
  m_pixmapUpdate=1;
  m_fromConstraints=1;
  resize(n_size);
  setPreferredSize(n_size);
  
};


void PlasmaTvGr::constraintsEvent(Plasma::Constraints constraints)
{
    calculateSizes();
    if (constraints & Plasma::FormFactorConstraint) {
        if (formFactor() == Plasma::Horizontal) {
            setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
	    setMinimumSize(128,36);
        }
        else if (formFactor() == Plasma::Vertical) {
            setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
	    setMinimumSize(36,128);
        }
        else {
            setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
	    setMinimumSize(64,64);
        }
    }
    //update();
}

void PlasmaTvGr::updateLists() //slot to be called every hour.
{
    bool updateQ=false;
    QString s1;
    //check date to see if update is needed
    m_date2=m_date2.currentDateTime();
    if(m_date2.time().hour()<5) m_date2=m_date2.addDays(-1); //remove one day if it is early in the morning
    s1=m_date2.toString("yyyyMMdd");
    if((m_date1.date()!=m_date2.date())||m_httpOK==false) updateQ=true; 
    s1="searchDt="+s1;
    kDebug()<< "updateQ:"<<updateQ<<"("<<m_date1<<","<<m_date2<<")";
    m_fromConstraints=1; // no effect
    if(updateQ==true)
    {
    m_fromConstraints=0; //effect on
    kDebug()<<"Let's do the update.";
    m_job = KIO::storedHttpPost(s1.toLatin1(),QString("http://tv.pathfinder.gr/search"),KIO::HideProgressInfo);
    KIO::StoredTransferJob* storedJob = static_cast<KIO::StoredTransferJob*>( m_job );
    storedJob->addMetaData("content-type", "Content-Type: application/x-www-form-urlencoded" );
    connect(m_job, SIGNAL(result(KJob*)),this ,SLOT(slotGotData(KJob*)));
    };
    selectChannel(m_channelID);
    m_pixmapUpdate=1;
    update();
}


void PlasmaTvGr::slotGotData(KJob* job)
{
  if(job && job->error())
  {
    m_httpOK=false;
    kDebug()<<"error fetching data:"<<job->errorString();
  }else
  {
    kDebug() << "fetching data ok:";
    m_list1.clear();
    m_list2.clear();
    m_list3.clear();
    m_list4.clear();
    QTextCodec *codec = QTextCodec::codecForName("ISO 8859-7"); 
    KIO::StoredTransferJob* const storedJob = static_cast<KIO::StoredTransferJob*>( job );
    m_webpage=codec->toUnicode(storedJob->data().data());
    //rebuild lists
    QString s1,s2;
    int w1=0;
    int w2;
    //qDebug()<<m_webpage;
    m_webpage.replace("<tr class=\"odd\">","<tr class=\"even\">");//make all odd = even
    m_webpage.replace("<tr class=\"now\">","<tr class=\"even\">");//make all now = even
    while((w1=m_webpage.indexOf("<tr class=\"even\">",w1))!=-1)
    {
	w2=m_webpage.indexOf("<tr class=\"even\">",w1+1);
	//FIXME known bug. the last entry does not end with <tr class so it is dropped
	if(w2==-1) break;
	s1=m_webpage.mid(w1,w2-w1);//this is the program line;
	QRegExp rxx1("<img.*>");rxx1.setMinimal(true);//remove the <img column
	s1.replace(rxx1,"");
	QRegExp rxx2("<a href.*>");rxx2.setMinimal(true);//remove the <a href tag
	s1.replace(rxx2,"");
	s1.replace("</a>",""); //and the ending </a>
	QRegExp rx("(>.*<)");
	rx.setMinimal(true);
	int pos=0;
	QStringList list1;
	while((pos=rx.indexIn(s1,pos))!=-1)
	{
		list1<<rx.cap(1);
		pos+=rx.matchedLength();
	};
	//qDebug()<<">>>>>>>>"<<list1.size()<<list1;
	if(list1.size()==8)
	{
	//m_list1 hour, m_list2 title, m_list3 station, m_list4 categ
	s2=list1[1];s2.remove(0,1);s2.chop(1);
	m_list1.append(s2);
	s2=list1[3];s2.remove(0,1);s2.chop(1);
	s2=s2.replace(QRegExp("&[a-zA-z&]*")," ");
	m_list2.append(s2);
	s2=list1[5];s2.remove(0,1);s2.chop(1);
	m_list3.append(s2);
	s2=list1[7];s2.remove(0,1);s2.chop(1);
	m_list4.append(s2);
	//kDebug()<<m_list1.last()<<m_list2.last()<<m_list3.last()<<m_list4.last();
	};
	++w1;
    };
    //qDebug()<<m_webpage;
    kDebug()<<"parsed"<<m_list1.size()<<"entries";
    selectChannel(m_channelID);
    m_httpOK=true;
    m_date1=m_date2;
    m_pixmapUpdate=1;
    update();
  };
  
};


void PlasmaTvGr::selectChannel(int i1)
{
    int w1,w2;
    int sameHour;
    QDateTime date1,date2;
    double k1,k2,k0;
    m_chan1.clear();//hour
    m_chan2.clear();//title
    m_chan3.clear();//station
    m_chan4.clear();//categ
    for(w1=0;w1<=m_list1.size()-1;w1++)
    {
      if(m_list3[w1]==chan[i1])
      {
	sameHour=0;//check if the same hour exists already for the specific station
	for(w2=0;w2<=m_chan1.size()-1;w2++)
	{
	  if(m_chan1[w2]==m_list1[w1]) sameHour=1;
	};
	if(sameHour==0)
	{
	m_chan1.append(m_list1[w1]);
	m_chan2.append(m_list2[w1]);
	m_chan3.append(m_list3[w1]);
	m_chan4.append(m_list4[w1]);
	};
      };
    };
    m_channelID=i1;
    if(m_chan1.size()==0) return; // { m_httpOK=false;return; }; //typical check for no crash the next line
    k0=m_chan1[0].left(2).toDouble()+m_chan1[0].right(2).toDouble()/60.0; // time of the first tv program of the channel
    date1=date1.currentDateTime();
    k1=date1.time().hour()+date1.time().minute()/60.0; // k1 is the current time in decimal hours
    if(k1<5) k1=k1+24.0;// tranlate hours after midnight by 24 hours so they are last
    w2=0;
    int isBreak=0;
    for(w1=0;w1<=m_chan1.size()-1;w1++)
    {
      k2=m_chan1[w1].left(2).toDouble()+m_chan1[w1].right(2).toDouble()/60.0;
      if(k2<k0) k2=k2+24.0;
      w2=w1;
      if(k2>k1) {isBreak=1;break;};
    };
    m_timeIndex=w2;
    if(m_timeIndex!=0 && isBreak!=0) m_timeIndex--;
    //kDebug()<<i1;
    //kDebug()<<"index in the Channel:"<<m_timeIndex<<"\n";
};


void PlasmaTvGr::showConfigurationInterface()
{
    initConfig();
    if(m_dialog!=0) m_dialog->show();
};

void PlasmaTvGr::configAccepted()
{
  if(m_dialog)
  {
  switch(ui.loopBox->currentIndex()){
  case 0:
    m_loop=0;break;
  case 1:
    m_loop=15;break;
  case 2:
    m_loop=30;break;
  case 3:
    m_loop=60;break;
  case 4:
    m_loop=600;break;
  case 5:
    m_loop=1800;break;
  default:
    m_loop=0;
  };
  m_fontSize=ui.fontBox->currentText().toInt();
  };
  m_chanList="";
  QListWidgetItem* item;
  int w1,w2;
  int shownOne=0;
  for(w1=0;w1<ui.list1->count();w1++)
  {
    item=ui.list1->item(w1);
    w2=item->isSelected();
    if(w2==1) shownOne=1;
    m_chanList.append(QString::number(w2)+",");
  };
  m_chanList.chop(1);
  if(shownOne==0) m_chanList[0]='1';
  KConfigGroup cg = config();
  cg.writeEntry("LoopSeconds", m_loop);
  cg.writeEntry("SelectedChannel", m_channelID);
  cg.writeEntry("FontSize",m_fontSize);
  cg.writeEntry("ChannelList",m_chanList);
  m_channel=chan[m_channelID];
  if(m_loop!=0) m_loopTimer->start(m_loop*1000); else m_loopTimer->stop();
  calculateSizes();
  //updateGeometry();
  m_pixmapUpdate=1;
  update();
  emit configNeedsSaving();
};



void PlasmaTvGr::paintInterface(QPainter *pp,
        const QStyleOptionGraphicsItem *option, const QRect &contentsRect)
{
    QPainter* p;
    //Q_UNUSED(option);
    if(m_pixmapUpdate==1){ //there is no animation so , it is really a gui update
    int w1;
    int w4,w5;
    QString s1;
    QFontMetrics metric(m_font);
    //now replace oldPixmap with new
    if(m_newPixmap!=0)
    {
    if(m_oldPixmap!=0) delete m_oldPixmap;
    m_oldPixmap=new QPixmap(size().width(),size().height());
    m_oldPixmap->fill(QColor(Qt::transparent));
    QPainter p1(m_oldPixmap);
    p1.drawPixmap(option->rect, m_newPixmap->copy(), option->rect);
    p1.end();
    };
    if(m_newPixmap!=0) delete m_newPixmap;
    m_newPixmap=new QPixmap(size().width(),size().height());
    m_newPixmap->fill(QColor(Qt::transparent));
    p=new QPainter(m_newPixmap);
    p->setRenderHint(QPainter::TextAntialiasing);
    //p->setRenderHint(QPainter::SmoothPixmapTransform);
    //p->setRenderHint(QPainter::Antialiasing);
    p->setClipRect(contentsRect,Qt::ReplaceClip);
    if(m_httpOK==true){
    setBusy(false);
    if (formFactor() == Plasma::Horizontal) {
      p->save();
      p->setPen(QPen(m_colfg));
      p->setFont(m_font);
      //p->drawRect(m_textBox);
      p->setOpacity(0.5);
      m_images[m_channelID]->paint(p,contentsRect.center().x()
			      -m_images[m_channelID]->size().width()/2.0,contentsRect.top(),"layer1");
      p->setOpacity(1.0);
      w4=m_timeIndex;
      w5=w4+m_numOfLines;
      if(w4==w5) w5++;
      if(w5>m_chan1.size()) w5=m_chan1.size();
      for(w1=w4;w1<=w5-1;w1++)
      {
	p->setPen(QColor(0,0,0));
	p->drawText(m_textBox.x()+1,m_textBox.y()+1+m_textH*(w1-w4+1),m_chan1[w1]+" "+m_chan2[w1]);
	if(w1==m_timeIndex) p->setPen(m_colselfg); else p->setPen(m_colfg);
	p->drawText(m_textBox.x(),m_textBox.y()+m_textH*(w1-w4+1),m_chan1[w1]+" "+m_chan2[w1]);
      };
      p->restore();
    };
    if (formFactor() == Plasma::Vertical) {
	p->save();
	p->setPen(QPen(m_colfg));
	p->setFont(m_font);
	//p->drawRect(m_textBox);
	p->rotate(-90);
	p->setOpacity(0.5);
	m_images[m_channelID]->paint(p,contentsRect.left()-contentsRect.height()/2
	      -m_images[m_channelID]->size().width()/2 ,contentsRect.top(),"layer1");
	p->setOpacity(1.0);
	w4=m_timeIndex;
	w5=w4+m_numOfLines;
	if(w4==w5) w5++;
	if(w5>m_chan1.size()) w5=m_chan1.size();
	for(w1=w4;w1<=w5-1;w1++)
	{
	  p->setPen(QColor(0,0,0));
	  p->drawText(m_textBox.x()-m_textBox.y()-m_textBox.height()+1, m_textBox.bottom()+m_textH*(w1-w4+1)-		  	m_textBox.y()-m_textBox.height()+1,m_chan1[w1]+" "+m_chan2[w1]);
	  if(w1==m_timeIndex) p->setPen(m_colselfg); else p->setPen(m_colfg);
	  p->drawText(m_textBox.x()-m_textBox.y()-m_textBox.height(), m_textBox.bottom()+m_textH*(w1-w4+1)-		  	m_textBox.y()-m_textBox.height(),m_chan1[w1]+" "+m_chan2[w1]);
	};
	p->restore();
        
	};
    if (formFactor() == Plasma::Planar) {
      p->save();
      p->setPen(QPen(m_colfg)); 
      p->setFont(m_font);
      m_images[m_channelID]->paint(p,contentsRect.center().x()
				-m_images[m_channelID]->size().width()/2.0,contentsRect.top(),"layer1");
      //p->drawRect(m_textBox);
      w4=m_timeIndex-1;
      if(w4<0) w4=0;
      w5=w4+m_numOfLines;
      if(w4==w5) w5++;
      if(w5>m_chan1.size()) w5=m_chan1.size();
      for(w1=w4;w1<=w5-1;w1++)
      {
	p->setPen(QColor(0,0,0));
        p->drawText(m_textBox.x()+1,m_textBox.y()+m_textH*(w1-w4+1)+1,m_chan1[w1]+"  "+m_chan2[w1]);
	if(w1==m_timeIndex) p->setPen(m_colselfg); else p->setPen(m_colfg);
	p->drawText(m_textBox.x(),m_textBox.y()+m_textH*(w1-w4+1),m_chan1[w1]+"  "+m_chan2[w1]);
      };
      p->restore();
      p->end();
     };

      }else
      {
	//no data available
	p->save();
	p->setFont(m_font);
	p->setPen(QPen(m_colfg));
	p->drawText(contentsRect, Qt::AlignVCenter | Qt::AlignHCenter, "No Data.");
	p->restore();
	p->end();
	delete p;
	setBusy(true);
      };
	m_pixmapUpdate=0;
	if(m_fromConstraints==0)
	{
	  const int AnimDuration=250;
	  m_animID=Plasma::Animator::self()->customAnimation(10,AnimDuration, Plasma::Animator::LinearCurve,this,"animationUpdate");
	  pp->drawPixmap(option->rect, m_oldPixmap->copy(), option->rect);
	}else 
	{
          m_fromConstraints=0;
	  pp->drawPixmap(option->rect, m_newPixmap->copy(), option->rect);
	};
    }else //m_pixmapUpdate==0
    {
      if(m_animID!=0)
      {
      pp->setOpacity(1.00-m_animOpacity);
      pp->drawPixmap(option->rect, m_oldPixmap->copy(), option->rect);
      pp->setOpacity(m_animOpacity);
      pp->drawPixmap(option->rect, m_newPixmap->copy(), option->rect);
      }else //no animation
      {
          pp->drawPixmap(option->rect, m_newPixmap->copy(), option->rect);
      };
    };
}


void PlasmaTvGr::animationUpdate(qreal progress)
{
    if(progress==1)
    {
      //m_animID=0;
      /*if(m_newPixmap!=0)
      {
      if(m_oldPixmap!=0) delete m_oldPixmap;
      m_oldPixmap=new QPixmap(size().width(),size().height());
      m_oldPixmap->fill(QColor(Qt::transparent));
      QPainter p1(m_oldPixmap);
      p1.drawPixmap(rect(), m_newPixmap->copy(), rect());
      p1.end();
      };*/

    };
    m_animOpacity=progress;
    update();
};

 
#include "plasma-tvgr.moc"
